@@ -27,12 +27,12 @@
 
 import os
 
-VERSION = "2.2.0"
-DATE = "20161220000000"
+VERSION = "2.3.1"
+DATE = "20170331000000"
 NAME = "openshot-qt"
 PRODUCT_NAME = "OpenShot Video Editor"
 GPL_VERSION = "3"
-DESCRIPTION = "Create and edit videos and movies"
+DESCRIPTION = "Create and edit stunning videos, movies, and animations"
 COMPANY_NAME = "OpenShot Studios, LLC"
 COPYRIGHT = "Copyright (c) 2008-2016 %s" % COMPANY_NAME
 CWD = os.getcwd()
@@ -41,6 +41,7 @@ HOME_PATH = os.path.join(os.path.expanduser("~"))
 USER_PATH = os.path.join(HOME_PATH, ".openshot_qt")
 BACKUP_PATH = os.path.join(USER_PATH, "backup")
 BLENDER_PATH = os.path.join(USER_PATH, "blender")
+ASSETS_PATH = os.path.join(USER_PATH, "assets")
 THUMBNAIL_PATH = os.path.join(USER_PATH, "thumbnail")
 CACHE_PATH = os.path.join(USER_PATH, "cache")
 PREVIEW_CACHE_PATH = os.path.join(USER_PATH, "preview-cache")
@@ -53,7 +54,7 @@ EXPORT_TESTS = os.path.join(USER_PATH, "tests")
 USER_PROFILES_PATH = os.path.join(USER_PATH, "profiles")
 
 # Create PATHS if they do not exist (this is where temp files are stored... such as cached thumbnails)
-for folder in [USER_PATH, THUMBNAIL_PATH, CACHE_PATH, BLENDER_PATH, TITLE_PATH, PROFILES_PATH, IMAGES_PATH,
+for folder in [USER_PATH, THUMBNAIL_PATH, CACHE_PATH, BLENDER_PATH, ASSETS_PATH, TITLE_PATH, PROFILES_PATH, IMAGES_PATH,
                TRANSITIONS_PATH, EXPORT_TESTS, BACKUP_PATH, USER_PROFILES_PATH, PREVIEW_CACHE_PATH]:
     if not os.path.exists(folder.encode("UTF-8")):
         os.makedirs(folder, exist_ok=True)
