@@ -131,8 +131,8 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                 # Get the matching item
                 obj = obj[key_part]
 
-        # After processing each key, we've found object, return copy of it
-        return copy.deepcopy(obj)
+        # After processing each key, we've found object, return it
+        return obj
 
     def set(self, key, value):
         """Prevent calling JsonDataStore set() method. It is not allowed in ProjectDataStore, as changes come from UpdateManager."""
@@ -333,6 +333,9 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
             # Convert all paths back to absolute
             self.convert_paths_to_absolute()
 
+            # Check if paths are all valid
+            self.check_if_paths_are_valid()
+
             # Copy any project thumbnails to main THUMBNAILS folder
             loaded_project_folder = os.path.dirname(self.current_filepath)
             project_thumbnails_folder = os.path.join(loaded_project_folder, "thumbnail")
@@ -351,6 +354,7 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
 
         # Get app, and distribute all project data through update manager
         from classes.app import get_app
+        get_app().processEvents()
         get_app().updates.load(self._data)
 
         # Clear needs save flag
@@ -391,6 +395,8 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
         from classes.legacy.openshot.classes import keyframe as legacy_keyframe
         from classes.legacy.openshot.classes import files as legacy_files
         from classes.legacy.openshot.classes import transition as legacy_transition
+        from classes.legacy.openshot.classes import effect as legacy_effect
+        from classes.legacy.openshot.classes import marker as legacy_marker
         sys.modules['openshot.classes'] = legacy_classes
         sys.modules['classes.project'] = legacy_project
         sys.modules['classes.sequences'] = legacy_sequences
@@ -399,6 +405,8 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
         sys.modules['classes.keyframe'] = legacy_keyframe
         sys.modules['classes.files'] = legacy_files
         sys.modules['classes.transition'] = legacy_transition
+        sys.modules['classes.effect'] = legacy_effect
+        sys.modules['classes.marker'] = legacy_marker
 
         # Keep track of files that failed to load
         failed_files = []
@@ -504,9 +512,9 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                             # Video Fade IN
                             if clip.video_fade_in:
                                 # Add keyframes
-                                start = openshot.Point(clip.start_time * fps_float, 0.0, openshot.BEZIER)
+                                start = openshot.Point(round(clip.start_time * fps_float) + 1, 0.0, openshot.BEZIER)
                                 start_object = json.loads(start.Json())
-                                end = openshot.Point((clip.start_time + clip.video_fade_in_amount) * fps_float, 1.0, openshot.BEZIER)
+                                end = openshot.Point(round((clip.start_time + clip.video_fade_in_amount) * fps_float) + 1, 1.0, openshot.BEZIER)
                                 end_object = json.loads(end.Json())
                                 new_clip["alpha"]["Points"].append(start_object)
                                 new_clip["alpha"]["Points"].append(end_object)
@@ -514,9 +522,9 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                             # Video Fade OUT
                             if clip.video_fade_out:
                                 # Add keyframes
-                                start = openshot.Point((clip.end_time - clip.video_fade_out_amount) * fps_float, 1.0, openshot.BEZIER)
+                                start = openshot.Point(round((clip.end_time - clip.video_fade_out_amount) * fps_float) + 1, 1.0, openshot.BEZIER)
                                 start_object = json.loads(start.Json())
-                                end = openshot.Point(clip.end_time * fps_float, 0.0, openshot.BEZIER)
+                                end = openshot.Point(round(clip.end_time * fps_float) + 1, 0.0, openshot.BEZIER)
                                 end_object = json.loads(end.Json())
                                 new_clip["alpha"]["Points"].append(start_object)
                                 new_clip["alpha"]["Points"].append(end_object)
@@ -532,9 +540,9 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                             # Audio Fade IN
                             if clip.audio_fade_in:
                                 # Add keyframes
-                                start = openshot.Point(clip.start_time * fps_float, 0.0, openshot.BEZIER)
+                                start = openshot.Point(round(clip.start_time * fps_float) + 1, 0.0, openshot.BEZIER)
                                 start_object = json.loads(start.Json())
-                                end = openshot.Point((clip.start_time + clip.video_fade_in_amount) * fps_float, clip.volume / 100.0, openshot.BEZIER)
+                                end = openshot.Point(round((clip.start_time + clip.video_fade_in_amount) * fps_float) + 1, clip.volume / 100.0, openshot.BEZIER)
                                 end_object = json.loads(end.Json())
                                 new_clip["volume"]["Points"].append(start_object)
                                 new_clip["volume"]["Points"].append(end_object)
@@ -542,9 +550,9 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                             # Audio Fade OUT
                             if clip.audio_fade_out:
                                 # Add keyframes
-                                start = openshot.Point((clip.end_time - clip.video_fade_out_amount) * fps_float, clip.volume / 100.0, openshot.BEZIER)
+                                start = openshot.Point(round((clip.end_time - clip.video_fade_out_amount) * fps_float) + 1, clip.volume / 100.0, openshot.BEZIER)
                                 start_object = json.loads(start.Json())
-                                end = openshot.Point(clip.end_time * fps_float, 0.0, openshot.BEZIER)
+                                end = openshot.Point(round(clip.end_time * fps_float) + 1, 0.0, openshot.BEZIER)
                                 end_object = json.loads(end.Json())
                                 new_clip["volume"]["Points"].append(start_object)
                                 new_clip["volume"]["Points"].append(end_object)
@@ -571,7 +579,7 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
 
                             brightness = openshot.Keyframe()
                             brightness.AddPoint(1, trans_begin_value, openshot.BEZIER)
-                            brightness.AddPoint(trans.length * fps_float, trans_end_value, openshot.BEZIER)
+                            brightness.AddPoint(round(trans.length * fps_float) + 1, trans_end_value, openshot.BEZIER)
                             contrast = openshot.Keyframe(trans.softness * 10.0)
 
                             # Create transition dictionary
@@ -728,26 +736,40 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                 path = file["path"]
 
                 # Find any temp BLENDER file paths
-                if info.BLENDER_PATH in path:
+                if info.BLENDER_PATH in path or info.ASSETS_PATH in path:
                     log.info("Temp blender file path detected in file")
 
                     # Get folder of file
                     folder_path, file_name = os.path.split(path)
                     parent_path, folder_name = os.path.split(folder_path)
-                    # Update path to new folder
-                    path = os.path.join(new_project_folder, folder_name)
-                    # Copy temp folder to project folder
-                    shutil.copytree(folder_path, path)
+                    new_parent_path = new_project_folder
+
+                    if os.path.isdir(path) or "%" in path:
+                        # Update path to new folder
+                        new_parent_path = os.path.join(new_project_folder, folder_name)
+
+                        # Copy blender tree into new folder
+                        shutil.copytree(folder_path, new_parent_path)
+                    else:
+                        # New path
+                        new_parent_path = os.path.join(new_project_folder, "assets")
+
+                        # Ensure blender folder exists
+                        if not os.path.exists(new_parent_path):
+                            os.mkdir(new_parent_path)
+
+                        # Copy titles/individual files into new folder
+                        shutil.copy2(path, os.path.join(new_parent_path, file_name))
 
                     # Update paths in project to new location
-                    file["path"] = os.path.join(path, file_name)
+                    file["path"] = os.path.join(new_parent_path, file_name)
 
             # Loop through each clip
             for clip in self._data["clips"]:
                 path = clip["reader"]["path"]
 
                 # Find any temp BLENDER file paths
-                if info.BLENDER_PATH in path:
+                if info.BLENDER_PATH in path or info.ASSETS_PATH in path:
                     log.info("Temp blender file path detected in clip")
 
                     # Get folder of file
@@ -764,7 +786,7 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
                 path = clip["image"]
 
                 # Find any temp BLENDER file paths
-                if info.BLENDER_PATH in path:
+                if info.BLENDER_PATH in path or info.ASSETS_PATH in path:
                     log.info("Temp blender file path detected in clip thumbnail")
 
                     # Get folder of file
@@ -801,6 +823,7 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
 
         # Save setting
         s.set("recent_projects", recent_projects)
+        s.save()
 
     def convert_paths_to_relative(self, file_path):
         """ Convert all paths relative to this filepath """
@@ -868,6 +891,55 @@ class ProjectDataStore(JsonDataStore, UpdateInterface):
             log.error("Error while converting absolute paths to relative paths: %s" % str(ex))
 
 
+    def check_if_paths_are_valid(self):
+        """Check if all paths are valid, and prompt to update them if needed"""
+        # Get project folder
+        starting_folder = None
+        if self.current_filepath:
+            starting_folder = os.path.dirname(self.current_filepath)
+
+        # Get translation method
+        from classes.app import get_app
+        _ = get_app()._tr
+
+        from PyQt5.QtWidgets import QFileDialog, QMessageBox
+
+        # Loop through each files (in reverse order)
+        for file in reversed(self._data["files"]):
+            path = file["path"]
+            parent_path, file_name_with_ext = os.path.split(path)
+            while not os.path.exists(path) and "%" not in path:
+                # File already exists! Prompt user to find missing file
+                QMessageBox.warning(None, _("Missing File (%s)") % file["id"], _("%s cannot be found.") % file_name_with_ext)
+                starting_folder = QFileDialog.getExistingDirectory(None, _("Find directory that contains: %s" % file_name_with_ext), starting_folder)
+                log.info("Missing folder chosen by user: %s" % starting_folder)
+                if starting_folder:
+                    # Update file path
+                    path = os.path.join(starting_folder, file_name_with_ext)
+                    file["path"] = path
+                else:
+                    log.info('Removed missing file: %s' % file_name_with_ext)
+                    self._data["files"].remove(file)
+                    break
+
+        # Loop through each clip (in reverse order)
+        for clip in reversed(self._data["clips"]):
+            path = clip["reader"]["path"]
+            parent_path, file_name_with_ext = os.path.split(path)
+            while not os.path.exists(path) and "%" not in path:
+                # Clip already exists! Prompt user to find missing file
+                QMessageBox.warning(None, _("Missing File in Clip (%s)") % clip["id"], _("%s cannot be found.") % file_name_with_ext)
+                starting_folder = QFileDialog.getExistingDirectory(None, _("Find directory that contains: %s" % file_name_with_ext), starting_folder)
+                log.info("Missing folder chosen by user: %s" % starting_folder)
+                if starting_folder:
+                    # Update clip path
+                    path = os.path.join(starting_folder, file_name_with_ext)
+                    clip["reader"]["path"] = path
+                else:
+                    log.info('Removed missing clip: %s' % file_name_with_ext)
+                    self._data["clips"].remove(clip)
+                    break
+
     def convert_paths_to_absolute(self):
         """ Convert all paths to absolute """
         try:
