@@ -242,6 +242,7 @@ App.controller('TimelineCtrl',function($scope) {
   $scope.snapline_position = 0.0;
   $scope.snapline = false;
   $scope.enable_snapping = true;
+  $scope.enable_razor = false;
   $scope.debug = false;
   $scope.min_width = 1024;
   $scope.track_label = "Track %s";
@@ -284,7 +285,7 @@ App.controller('TimelineCtrl',function($scope) {
   $scope.PreviewFrame = function(position_seconds) {
 	  // Determine frame
 	  var frames_per_second = $scope.project.fps.num / $scope.project.fps.den;
-	  var frame = (position_seconds * frames_per_second) + 1;
+	  var frame = Math.round(position_seconds * frames_per_second) + 1;
 	  
 	  // Update GUI with position (to the preview can be updated)
 	  if ($scope.Qt)
@@ -296,7 +297,7 @@ App.controller('TimelineCtrl',function($scope) {
   $scope.PreviewClipFrame = function(clip_id, position_seconds) {
 	  // Determine frame
 	  var frames_per_second = $scope.project.fps.num / $scope.project.fps.den;
-	  var frame = (position_seconds * frames_per_second) + 1;
+	  var frame = Math.round(position_seconds * frames_per_second) + 1;
 
 	  // Update GUI with position (to the preview can be updated)
 	  if ($scope.Qt)
@@ -309,8 +310,8 @@ App.controller('TimelineCtrl',function($scope) {
   	keyframes = {};
 
     var frames_per_second = $scope.project.fps.num / $scope.project.fps.den;
-    var clip_start_x = Math.round(object.start * frames_per_second) + 1.0;
-    var clip_end_x = Math.round(object.end * frames_per_second) + 1.0;
+    var clip_start_x = Math.round(object.start * frames_per_second) + 1;
+    var clip_end_x = Math.round(object.end * frames_per_second) + 1;
 
  	// Loop through properties of an object (clip/transition), looking for keyframe points
 	for (child in object) {
@@ -392,11 +393,28 @@ App.controller('TimelineCtrl',function($scope) {
 // ############# QT FUNCTIONS #################### //
 
  // Change the scale and apply to scope
- $scope.setScale = function(scaleVal){
+ $scope.setScale = function(scaleVal, cursor_x){
+
+	  // Get scrollbar positions
+	  var horz_scroll_offset = $("#scrolling_tracks").scrollLeft();
+	  var track_labels_width = $("#track_controls").width();
+
+	  // Determine actual x coordinate (over timeline)
+      var center_x = Math.max(cursor_x - track_labels_width, 0);
+      if (cursor_x == 0)
+      	center_x = 0;
+
+      // Determine time of cursor position
+      var cursor_time = parseFloat(center_x + horz_scroll_offset) / $scope.pixelsPerSecond;
+
       $scope.$apply(function(){
-         $scope.project.scale = scaleVal;
-         $scope.pixelsPerSecond =  parseFloat($scope.project.tick_pixels) / parseFloat($scope.project.scale);
-     });
+         $scope.project.scale = parseFloat(scaleVal);
+         $scope.pixelsPerSecond = parseFloat($scope.project.tick_pixels) / parseFloat($scope.project.scale);
+      });
+
+	 // Scroll back to correct cursor time (minus the difference of the cursor location)
+	 var new_cursor_x = Math.round((cursor_time * $scope.pixelsPerSecond) - center_x);
+	 $("#scrolling_tracks").scrollLeft(new_cursor_x);
  };
 
  // Set the audio data for a clip
@@ -466,6 +484,13 @@ App.controller('TimelineCtrl',function($scope) {
      });
  };
 
+ // Change the razor mode
+ $scope.SetRazorMode = function(enable_razor){
+      $scope.$apply(function(){
+         $scope.enable_razor = enable_razor;
+     });
+ };
+
  // Get the color of an effect
  $scope.GetEffectColor = function(effect_type){
 	switch (effect_type)
@@ -583,6 +608,16 @@ App.controller('TimelineCtrl',function($scope) {
 		$scope.SelectEffect("", clear_selections);
 	}
 
+	// Call slice method and exit (don't actually select the clip)
+	if (id != "" && $scope.enable_razor) {
+        if ($scope.Qt) {
+			cursor_seconds = $scope.GetJavaScriptPosition(event.clientX)
+            timeline.RazorSliceAtCursor(id, "", cursor_seconds)
+        }
+		// Don't actually select clip
+        return;
+    }
+
 	// Is CTRL pressed?
 	is_ctrl = false;
 	if (event && event.ctrlKey)
@@ -613,6 +648,16 @@ App.controller('TimelineCtrl',function($scope) {
 		$scope.SelectEffect("", true);
 	}
 
+	// Call slice method and exit (don't actually select the transition)
+	if (id != "" && $scope.enable_razor) {
+        if ($scope.Qt) {
+			cursor_seconds = $scope.GetJavaScriptPosition(event.clientX)
+            timeline.RazorSliceAtCursor("", id, cursor_seconds)
+        }
+		// Don't actually select transition
+        return;
+    }
+
 	// Is CTRL pressed?
 	is_ctrl = false;
 	if (event && event.ctrlKey)
@@ -1042,44 +1087,50 @@ $scope.SetTrackLabel = function (label){
 
 	// Loop through each pixel position (supports multiple positions: i.e. left and right side of bounding box)
 	for (var pos_index = 0; pos_index < pixel_positions.length; pos_index++) {
-		var pixel_position = pixel_positions[pos_index];
-		var position = pixel_position / $scope.pixelsPerSecond;
-		 
+		var position = pixel_positions[pos_index];
+
 		// Add clip positions to array
 		for (var index = 0; index < $scope.project.clips.length; index++) {
 			var clip = $scope.project.clips[index];
+			var clip_left_position = clip.position * $scope.pixelsPerSecond;
+			var clip_right_position = (clip.position + (clip.end - clip.start)) * $scope.pixelsPerSecond;
 
 			// exit out if this item is in ignore_ids
 			if (ignore_ids.hasOwnProperty(clip.id))
 				continue;
 			
-			diffs.push({'diff' : position - clip.position, 'position' : clip.position}, // left side of clip
-			           {'diff' : position - (clip.position + (clip.end - clip.start)), 'position' : clip.position + (clip.end - clip.start)}); // right side of clip
+			diffs.push({'diff' : position - clip_left_position, 'position' : clip_left_position}, // left side of clip
+			           {'diff' : position - clip_right_position, 'position' : clip_right_position}); // right side of clip
 		}
 		
 		// Add transition positions to array
 		for (var index = 0; index < $scope.project.effects.length; index++) {
 			var transition = $scope.project.effects[index];
+			var tran_left_position = transition.position * $scope.pixelsPerSecond;
+			var tran_right_position = (transition.position + (transition.end - transition.start)) * $scope.pixelsPerSecond;
+
 
 			// exit out if this item is in ignore_ids
 			if (ignore_ids.hasOwnProperty(transition.id))
 				continue;
 			
-			diffs.push({'diff' : position - transition.position, 'position' : transition.position}, // left side of transition
-			           {'diff' : position - (transition.position + (transition.end - transition.start)), 'position' : transition.position + (transition.end - transition.start)}); // right side of transition
+			diffs.push({'diff' : position - tran_left_position, 'position' : tran_left_position}, // left side of transition
+			           {'diff' : position - tran_right_position, 'position' : tran_right_position}); // right side of transition
 		}
 
 		// Add marker positions to array
 		for (var index = 0; index < $scope.project.markers.length; index++) {
 			var marker = $scope.project.markers[index];
+			var marker_position = marker.position * $scope.pixelsPerSecond;
 
-			diffs.push({'diff' : position - marker.position, 'position' : marker.position}, // left side of marker
-			           {'diff' : position - (marker.position + (marker.end - marker.start)), 'position' : marker.position + (marker.end - marker.start)}); // right side of marker
+			diffs.push({'diff' : position - marker_position, 'position' : marker_position}, // left side of marker
+			           {'diff' : position - marker_position, 'position' : marker_position}); // right side of marker
 		}
 
 		// Add playhead position to array
-		var playhead_diff = position - $scope.project.playhead_position;
-		diffs.push({'diff' : playhead_diff, 'position' : $scope.project.playhead_position });
+		var playhead_pixel_position = $scope.project.playhead_position * $scope.pixelsPerSecond;
+		var playhead_diff = position - playhead_pixel_position;
+		diffs.push({'diff' : playhead_diff, 'position' : playhead_pixel_position });
 		
 		// Loop through diffs (and find the smallest one)
 		for (var diff_index = 0; diff_index < diffs.length; diff_index++) {
@@ -1155,6 +1206,29 @@ $scope.SetTrackLabel = function (label){
 			return "track";
  };
 
+ // Determine which CSS classes are used on a clip
+ $scope.getClipStyle = function(clip){
+
+ 		style = "";
+		if (clip.selected)
+			style += "ui-selected ";
+
+		if ($scope.enable_razor)
+			style += "razor_cursor ";
+
+	 	return style;
+ };
+
+ // Determine which CSS classes are used on a clip label
+ $scope.getClipLabelStyle = function(clip){
+
+ 		style = "";
+		if ($scope.enable_razor)
+			style += "razor_cursor";
+
+	 	return style;
+ };
+
  // Apply JSON diff from UpdateManager (this is how the user interface communicates changes
  // to the timeline. A change can be an insert, update, or delete. The change is passed in
  // as JSON, which represents the change.
@@ -1313,6 +1387,12 @@ $scope.SetTrackLabel = function (label){
 
 	 // Lock / unlock any items
 	 $scope.LockItems();
+
+	 // Scroll to top/left when loading a project
+	 $("#scrolling_tracks").animate({
+		scrollTop: 0,
+		scrollLeft: 0
+	 }, 'slow');
 	 
 	 // return true
 	 return true;
