@@ -28,6 +28,7 @@
  """
 
 import os
+import sys
 import platform
 import shutil
 import webbrowser
@@ -77,6 +78,11 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
     RecoverBackup = pyqtSignal()
     FoundVersionSignal = pyqtSignal(str)
     WaveformReady = pyqtSignal(str, list)
+    TransformSignal = pyqtSignal(str)
+    ExportStarted = pyqtSignal(str, int, int)
+    ExportFrame = pyqtSignal(str, int, int, int)
+    ExportEnded = pyqtSignal(str)
+    MaxSizeChanged = pyqtSignal(object)
 
     # Save window settings on close
     def closeEvent(self, event):
@@ -109,15 +115,20 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
 
         # Stop threads
         self.StopSignal.emit()
+
+        # Process any queued events
+        QCoreApplication.processEvents()
+
+        # Stop preview thread (and wait for it to end)
+        self.preview_thread.player.CloseAudioDevice()
         self.preview_thread.kill()
+        self.preview_parent.background.exit()
+        self.preview_parent.background.wait(5000)
 
         # Close & Stop libopenshot logger
         openshot.ZmqLogger.Instance().Close()
         get_app().logger_libopenshot.kill()
 
-        # Wait for thread
-        self.preview_parent.background.wait(500)
-
         # Destroy lock file
         self.destroy_lock_file()
 
@@ -146,6 +157,11 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
             msg.setText(_("Your most recent unsaved project has been recovered."))
             msg.exec_()
 
+        else:
+            # No backup project found
+            # Load a blank project (to propagate the default settings)
+            get_app().project.load("")
+
     def create_lock_file(self):
         """Create a lock file"""
         lock_path = os.path.join(info.USER_PATH, ".lock")
@@ -160,9 +176,10 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
             found_stack = False
             log_start_counter = 0
             if os.path.exists(log_path):
-                with open(log_path, "r") as f:
+                with open(log_path, "rb") as f:
                     # Read from bottom up
-                    for line in reversed(f.readlines()):
+                    for raw_line in reversed(self.tail_file(f, 500)):
+                        line = str(raw_line, 'utf-8')
                         # Detect stack trace
                         if "End of Stack Trace" in line:
                             found_stack = True
@@ -242,7 +259,40 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
             # Remove file
             os.remove(lock_path)
 
+    def tail_file(self, f, n, offset=None):
+        """Read the end of a file (n number of lines)"""
+        avg_line_length = 90
+        to_read = n + (offset or 0)
+
+        while True:
+            try:
+                # Seek to byte position
+                f.seek(-(avg_line_length * to_read), 2)
+            except IOError:
+                # Byte position not found
+                f.seek(0)
+            pos = f.tell()
+            lines = f.read().splitlines()
+            if len(lines) >= to_read or pos == 0:
+                # Return the lines
+                return lines[-to_read:offset and -offset or None]
+            avg_line_length *= 2
+
     def actionNew_trigger(self, event):
+
+        app = get_app()
+        _ = app._tr  # Get translation function
+
+        # Do we have unsaved changes?
+        if get_app().project.needs_save():
+            ret = QMessageBox.question(self, _("Unsaved Changes"), _("Save changes to project first?"), QMessageBox.Cancel | QMessageBox.No | QMessageBox.Yes)
+            if ret == QMessageBox.Yes:
+                # Save project
+                self.actionSave_trigger(event)
+            elif ret == QMessageBox.Cancel:
+                # User canceled prompt
+                return
+
         # Clear any previous thumbnails
         self.clear_all_thumbnails()
 
@@ -293,6 +343,42 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         else:
             log.info('title editor add cancelled')
 
+    def actionEditTitle_trigger(self, event):
+
+        # Get selected svg title file
+        selected_file_id = self.selected_files[0]
+        file = File.get(id=selected_file_id)
+        file_path = file.data.get("path")
+
+        # Delete thumbnail for this file (it will be recreated soon)
+        thumb_path = os.path.join(info.THUMBNAIL_PATH, "{}.png".format(file.id))
+
+        # Check if thumb exists (and delete it)
+        if os.path.exists(thumb_path):
+            os.remove(thumb_path)
+
+        # show dialog for editing title
+        from windows.title_editor import TitleEditor
+        win = TitleEditor(file_path)
+        # Run the dialog event loop - blocking interaction on this window during that time
+        result = win.exec_()
+
+        # Force update of files model (which will rebuild missing thumbnails)
+        get_app().window.filesTreeView.refresh_view()
+
+    def actionDuplicateTitle_trigger(self, event):
+
+        # Get selected svg title file
+        selected_file_id = self.selected_files[0]
+        file = File.get(id=selected_file_id)
+        file_path = file.data.get("path")
+
+        # show dialog for editing title
+        from windows.title_editor import TitleEditor
+        win = TitleEditor(file_path, duplicate=True)
+        # Run the dialog event loop - blocking interaction on this window during that time
+        result = win.exec_()
+
     def actionImportImageSequence_trigger(self, event):
         # show dialog
         from windows.Import_image_seq import ImportImageSeq
@@ -386,7 +472,15 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
                 # Create blender folder
                 os.mkdir(info.BLENDER_PATH)
 
-            # Clear any blender animations
+            # Clear any assets folder
+            if os.path.exists(info.ASSETS_PATH):
+                log.info("Clear all assets: %s" % info.ASSETS_PATH)
+                # Remove assets folder
+                shutil.rmtree(info.ASSETS_PATH)
+                # Create assets folder
+                os.mkdir(info.ASSETS_PATH)
+
+            # Clear any backups
             if os.path.exists(info.BACKUP_PATH):
                 log.info("Clear all backups: %s" % info.BACKUP_PATH)
                 # Remove backup folder
@@ -402,6 +496,18 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         recommended_path = app.project.current_filepath
         if not recommended_path:
             recommended_path = info.HOME_PATH
+
+        # Do we have unsaved changes?
+        if get_app().project.needs_save():
+            ret = QMessageBox.question(self, _("Unsaved Changes"), _("Save changes to project first?"), QMessageBox.Cancel | QMessageBox.No | QMessageBox.Yes)
+            if ret == QMessageBox.Yes:
+                # Save project
+                self.actionSave_trigger(event)
+            elif ret == QMessageBox.Cancel:
+                # User canceled prompt
+                return
+
+        # Prompt for open project file
         file_path, file_type = QFileDialog.getOpenFileName(self, _("Open Project..."), recommended_path, _("OpenShot Project (*.osp)"))
 
         # Load project file
@@ -590,7 +696,7 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
 
     def actionHelpContents_trigger(self, event):
         try:
-            webbrowser.open("http://openshotusers.com/?app-menu")
+            webbrowser.open("http://www.openshot.org/files/user-guide/?app-menu")
             log.info("Help Contents is open")
         except:
             QMessageBox.information(self, "Error !",
@@ -680,19 +786,21 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         """ Preview the selected media file """
         log.info('actionPreview_File_trigger')
 
-        if self.selected_files:
+        # Loop through selected files (set 1 selected file if more than 1)
+        f = None
+        for file_id in self.selected_files:
             # Find matching file
-            f = File.get(id=self.selected_files[0])
-            if f:
-                # Get file path
-                previewPath = f.data["path"]
+            f = File.get(id=file_id)
 
-                # Load file into player
-                self.LoadFileSignal.emit(previewPath)
+        # Bail out if no file selected
+        if not f:
+            log.info(self.selected_files)
+            return
 
-                # Trigger play button
-                self.actionPlay.setChecked(False)
-                self.actionPlay.trigger()
+        # show dialog
+        from windows.cutting import Cutting
+        win = Cutting(f, preview=True)
+        win.show()
 
     def previewFrame(self, position_seconds, position_frames, time_code):
         """Preview a specific frame"""
@@ -833,6 +941,13 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         # Enable / Disable snapping mode
         self.timeline.SetSnappingMode(self.actionSnappingTool.isChecked())
 
+    def actionRazorTool_trigger(self, event):
+        """Toggle razor tool on and off"""
+        log.info('actionRazorTool_trigger')
+
+        # Enable / Disable razor mode
+        self.timeline.SetRazorMode(self.actionRazorTool.isChecked())
+
     def actionAddMarker_trigger(self, event):
         log.info("actionAddMarker_trigger")
 
@@ -858,14 +973,33 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         fps = get_app().project.get(["fps"])
         fps_float = float(fps["num"]) / float(fps["den"])
         current_position = self.preview_thread.current_frame / fps_float
+        all_marker_positions = []
+
+        # Get list of marker and important positions (like selected clip bounds)
+        for marker in Marker.filter():
+            all_marker_positions.append(marker.data["position"])
+
+        # Loop through selected clips (and add key positions)
+        for clip_id in self.selected_clips:
+            # Get selected object
+            selected_clip = Clip.get(id=clip_id)
+            if selected_clip:
+                all_marker_positions.append(selected_clip.data["position"])
+                all_marker_positions.append(selected_clip.data["position"] + (selected_clip.data["end"] - selected_clip.data["start"]))
+
+        # Loop through selected transitions (and add key positions)
+        for tran_id in self.selected_transitions:
+            # Get selected object
+            selected_tran = Transition.get(id=tran_id)
+            if selected_tran:
+                all_marker_positions.append(selected_tran.data["position"])
+                all_marker_positions.append(selected_tran.data["position"] + (selected_tran.data["end"] - selected_tran.data["start"]))
 
         # Loop through all markers, and find the closest one to the left
         closest_position = None
-        for marker in Marker.filter():
-            marker_position = marker.data["position"]
-
+        for marker_position in sorted(all_marker_positions):
             # Is marker smaller than position?
-            if marker_position < current_position:
+            if marker_position < current_position and (abs(marker_position - current_position) > 0.1):
                 # Is marker larger than previous marker
                 if closest_position and marker_position > closest_position:
                     # Set a new closest marker
@@ -875,11 +1009,15 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
                     closest_position = marker_position
 
         # Seek to marker position (if any)
-        if closest_position:
+        if closest_position != None:
             # Seek
-            frame_to_seek = int(closest_position * fps_float)
+            frame_to_seek = round(closest_position * fps_float) + 1
             self.SeekSignal.emit(frame_to_seek)
 
+            # Update the preview and reselct current frame in properties
+            get_app().window.refreshFrameSignal.emit()
+            get_app().window.propertyTableView.select_frame(frame_to_seek)
+
     def actionNextMarker_trigger(self, event):
         log.info("actionNextMarker_trigger")
         log.info(self.preview_thread.current_frame)
@@ -888,14 +1026,33 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         fps = get_app().project.get(["fps"])
         fps_float = float(fps["num"]) / float(fps["den"])
         current_position = self.preview_thread.current_frame / fps_float
+        all_marker_positions = []
+
+        # Get list of marker and important positions (like selected clip bounds)
+        for marker in Marker.filter():
+            all_marker_positions.append(marker.data["position"])
+
+        # Loop through selected clips (and add key positions)
+        for clip_id in self.selected_clips:
+            # Get selected object
+            selected_clip = Clip.get(id=clip_id)
+            if selected_clip:
+                all_marker_positions.append(selected_clip.data["position"])
+                all_marker_positions.append(selected_clip.data["position"] + (selected_clip.data["end"] - selected_clip.data["start"]))
+
+        # Loop through selected transitions (and add key positions)
+        for tran_id in self.selected_transitions:
+            # Get selected object
+            selected_tran = Transition.get(id=tran_id)
+            if selected_tran:
+                all_marker_positions.append(selected_tran.data["position"])
+                all_marker_positions.append(selected_tran.data["position"] + (selected_tran.data["end"] - selected_tran.data["start"]))
 
         # Loop through all markers, and find the closest one to the right
         closest_position = None
-        for marker in Marker.filter():
-            marker_position = marker.data["position"]
-
+        for marker_position in sorted(all_marker_positions):
             # Is marker smaller than position?
-            if marker_position > current_position:
+            if marker_position > current_position and (abs(marker_position - current_position) > 0.1):
                 # Is marker larger than previous marker
                 if closest_position and marker_position < closest_position:
                     # Set a new closest marker
@@ -905,11 +1062,15 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
                     closest_position = marker_position
 
         # Seek to marker position (if any)
-        if closest_position:
+        if closest_position != None:
             # Seek
-            frame_to_seek = int(closest_position * fps_float)
+            frame_to_seek = round(closest_position * fps_float) + 1
             self.SeekSignal.emit(frame_to_seek)
 
+            # Update the preview and reselct current frame in properties
+            get_app().window.refreshFrameSignal.emit()
+            get_app().window.propertyTableView.select_frame(frame_to_seek)
+
     def getShortcutByName(self, setting_name):
         """ Get a key sequence back from the setting name """
         s = settings.get_settings()
@@ -927,8 +1088,6 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
 
     def keyPressEvent(self, event):
         """ Process key press events and match with known shortcuts"""
-        MOD_MASK = (Qt.CTRL | Qt.ALT | Qt.SHIFT | Qt.META)
-
         # Detect the current KeySequence pressed (including modifier keys)
         key_value = event.key()
         print(key_value)
@@ -1579,6 +1738,9 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
             elif item_type == "effect":
                 self.selected_effects.clear()
 
+            # Clear transform (if any)
+            self.TransformSignal.emit("")
+
         if item_id:
             # If item_id is not blank, store it
             if item_type == "clip" and item_id not in self.selected_clips:
@@ -1589,27 +1751,39 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
                 self.selected_effects.append(item_id)
 
             # Change selected item in properties view
-            self.propertyTableView.loadProperties.emit(item_id, item_type)
+            self.show_property_id = item_id
+            self.show_property_type = item_type
+            self.show_property_timer.start()
 
     # Remove from the selected items
     def removeSelection(self, item_id, item_type):
-        if item_type == "clip" and item_id in self.selected_clips:
-            self.selected_clips.remove(item_id)
-        elif item_type == "transition" and item_id in self.selected_transitions:
-            self.selected_transitions.remove(item_id)
-        elif item_type == "effect" and item_id in self.selected_effects:
-            self.selected_effects.remove(item_id)
+        # Remove existing selection (if any)
+        if item_id:
+            if item_type == "clip" and item_id in self.selected_clips:
+                self.selected_clips.remove(item_id)
+            elif item_type == "transition" and item_id in self.selected_transitions:
+                self.selected_transitions.remove(item_id)
+            elif item_type == "effect" and item_id in self.selected_effects:
+                self.selected_effects.remove(item_id)
+
+        # Clear transform (if any)
+        get_app().window.TransformSignal.emit("")
 
         # Move selection to next selected clip (if any)
+        self.show_property_id = ""
+        self.show_property_type = ""
         if item_type == "clip" and self.selected_clips:
-            self.propertyTableView.loadProperties.emit(self.selected_clips[0], item_type)
+            self.show_property_id = self.selected_clips[0]
+            self.show_property_type = item_type
         elif item_type == "transition" and self.selected_transitions:
-            self.propertyTableView.loadProperties.emit(self.selected_transitions[0], item_type)
+            self.show_property_id = self.selected_transitions[0]
+            self.show_property_type = item_type
         elif item_type == "effect" and self.selected_effects:
-            self.propertyTableView.loadProperties.emit(self.selected_effects[0], item_type)
-        else:
-            # Clear selection in properties view
-            self.propertyTableView.loadProperties.emit("", "")
+            self.show_property_id = self.selected_effects[0]
+            self.show_property_type = item_type
+
+        # Change selected item in properties view
+        self.show_property_timer.start()
 
     # Update window settings in setting store
     def save_settings(self):
@@ -1754,17 +1928,21 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
 
         # rest of options
         self.timelineToolbar.addAction(self.actionSnappingTool)
+        self.timelineToolbar.addAction(self.actionRazorTool)
         self.timelineToolbar.addSeparator()
         self.timelineToolbar.addAction(self.actionAddMarker)
         self.timelineToolbar.addAction(self.actionPreviousMarker)
         self.timelineToolbar.addAction(self.actionNextMarker)
         self.timelineToolbar.addSeparator()
 
+        # Get project's initial zoom value
+        initial_scale = get_app().project.get(["scale"]) or 20
+
         # Setup Zoom slider
         self.sliderZoom = QSlider(Qt.Horizontal, self)
         self.sliderZoom.setPageStep(2)
         self.sliderZoom.setRange(1, 800)
-        self.sliderZoom.setValue(20)
+        self.sliderZoom.setValue(initial_scale)
         self.sliderZoom.setInvertedControls(True)
         self.sliderZoom.resize(100, 16)
 
@@ -1824,6 +2002,15 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
 
         return False
 
+    def show_property_timeout(self):
+        """Callback for show property timer"""
+
+        # Stop timer
+        self.show_property_timer.stop()
+
+        # Emit load properties signal
+        self.propertyTableView.loadProperties.emit(self.show_property_id, self.show_property_type)
+
     def InitKeyboardShortcuts(self):
         """Initialize all keyboard shortcuts from the settings file"""
 
@@ -1870,18 +2057,63 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         # Update cache reference, so it doesn't go out of scope
         self.cache_object = new_cache_object
 
+    def FrameExported(self, path, start_frame, end_frame, current_frame):
+        """Connect to Unity launcher (for Linux)"""
+        try:
+            if sys.platform == "linux" and self.has_launcher:
+                if not self.unity_launchers:
+                    # Get launcher only once
+                    from gi.repository import Unity
+                    self.unity_launchers.append(Unity.LauncherEntry.get_for_desktop_id("openshot-qt.desktop"))
+                    self.unity_launchers.append(Unity.LauncherEntry.get_for_desktop_id("appimagekit-openshot-qt.desktop"))
+
+                # Set progress and show progress bar
+                for launcher in self.unity_launchers:
+                    launcher.set_property("progress", current_frame / (end_frame - start_frame))
+                    launcher.set_property("progress_visible", True)
+
+        except:
+            # Just ignore
+            self.has_launcher = False
+
+    def ExportFinished(self, path):
+        """Export has completed"""
+        try:
+            if sys.platform == "linux" and self.has_launcher:
+                for launcher in self.unity_launchers:
+                    # Set progress on Unity launcher and hide progress bar
+                    launcher.set_property("progress", 0.0)
+                    launcher.set_property("progress_visible", False)
+        except:
+            pass
+
+
     def __init__(self, mode=None):
 
         # Create main window base class
         QMainWindow.__init__(self)
         self.mode = mode    # None or unittest (None is normal usage)
+        self.initialized = False
 
         # set window on app for reference during initialization of children
         get_app().window = self
         _ = get_app()._tr
 
+        # Load user settings for window
+        s = settings.get_settings()
+        self.recent_menu = None
+
         # Track metrics
         track_metric_session()  # start session
+
+        # Set unique install id (if blank)
+        if not s.get("unique_install_id"):
+            s.set("unique_install_id", str(uuid4()))
+
+            # Track 1st launch metric
+            track_metric_screen("initial-launch-screen")
+
+        # Track main screen
         track_metric_screen("main-screen")
 
         # Create blank tutorial manager
@@ -1893,10 +2125,6 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         # Set all keyboard shortcuts from the settings file
         self.InitKeyboardShortcuts()
 
-        # Load user settings for window
-        s = settings.get_settings()
-        self.recent_menu = None
-
         # Init UI
         ui_util.init_ui(self)
 
@@ -1930,6 +2158,7 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         else:
             self.filesTreeView = FilesListView(self)
         self.tabFiles.layout().addWidget(self.filesTreeView)
+        self.filesTreeView.setFocus()
 
         # Setup transitions tree
         if s.get("transitions_view") == "details":
@@ -1945,6 +2174,10 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
             self.effectsTreeView = EffectsListView(self)
         self.tabEffects.layout().addWidget(self.effectsTreeView)
 
+        # Process events before continuing
+        # TODO: Figure out why this is needed for a backup recovery to correctly show up on the timeline
+        get_app().processEvents()
+
         # Setup properties table
         self.txtPropertyFilter.setPlaceholderText(_("Filter"))
         self.propertyTableView = PropertiesTableView(self)
@@ -1955,6 +2188,16 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         # Init selection containers
         self.clearSelections()
 
+        # Show Property timer
+        # Timer to use a delay before showing properties (to prevent a mass selection from trying
+        # to update the property model hundreds of times)
+        self.show_property_id = None
+        self.show_property_type = None
+        self.show_property_timer = QTimer()
+        self.show_property_timer.setInterval(100)
+        self.show_property_timer.timeout.connect(self.show_property_timeout)
+        self.show_property_timer.stop()
+
         # Setup video preview QWidget
         self.videoPreview = VideoWidget()
         self.tabVideo.layout().insertWidget(0, self.videoPreview)
@@ -1991,5 +2234,21 @@ class MainWindow(QMainWindow, updates.UpdateWatcher, updates.UpdateInterface):
         # Create tutorial manager
         self.tutorial_manager = TutorialManager(self)
 
+        # Connect to Unity DBus signal (if linux)
+        if sys.platform == "linux":
+            self.unity_launchers = []
+            self.has_launcher = True
+            self.ExportFrame.connect(self.FrameExported)
+            self.ExportEnded.connect(self.ExportFinished)
+
         # Install event filter
-        self.installEventFilter(self)
\ No newline at end of file
+        self.installEventFilter(self)
+
+        # Save settings
+        s.save()
+
+        # Refresh frame
+        QTimer.singleShot(100, self.refreshFrameSignal.emit)
+
+        # Main window is initialized
+        self.initialized = True
\ No newline at end of file
